@@ -1,16 +1,22 @@
 #! /usr/bin/env python3
 # -*- coding: utf-8 -*-
 import traceback
+import  re
 from typing import Iterable
 import pymysql
 from pymysql.cursors import DictCursor
-from DBUtils.PooledDB import PooledDB
+try:
+    from dbutils.pooled_db import PooledDB
+except:
+    from DBUtils.PooledDB import PooledDB
+
 import time
 import random
 import uuid
 import logging
 import traceback
-from sqlgenerator import SqlGenerator
+from .sqlgenerator import SqlGenerator, MysqlSqler
+import psycopg2 as pg2
 
 import logging
 logger = logging.getLogger(__name__)
@@ -22,15 +28,56 @@ handler.setFormatter(formatter)
 logger.addHandler(handler)
 
 
+class SqlHandler(object):
+
+
+        
+
+    def get_connection(self):
+        raise NotImplementedError("get_connection function should be called on child object ")
+
+    def get_cursor(self):
+        raise NotImplementedError("get_cursor function should be called on child object ")
+
+
 class SimpleConnector(SqlGenerator):
 
     logger = logger
 
     def __init__(self, **kwargs):
+        if kwargs.get('string', None):
+            connargs = self.get_conn_args_from_str(kwargs['string'])
+        
         connargs = {'user': 'root', 'port': 3306, 'charset': 'utf8'}
+        
+            
         connargs.update(kwargs)
         self.connector = pymysql.connect(**connargs)
 
+    def get_conn_args_from_str(self, string):
+        """
+        @param string,connect args on terminal ,for example: mysql -h127.0.0.1 -p1234  -uroot -p123456 -Dtest_base 
+        return a dict with connect args
+        { "host":"127.0.0.1","port":1234,"password":"123456","db":"test_base"  }
+        """
+        conn_args = dict()
+        patter_dict = {
+            "host": "-h([\d\.]+)",
+            "port": "-P(\d+)",
+            "user": "-u(\S+)",
+            "password": "-p(\S+)",
+            "db":"-D(\S+)",
+        }
+        for k, v in patter_dict.items():
+            result = re.findall(v, string)
+            number=len(result)
+            if number==1:
+                conn_args[k] = result[0]
+            else:
+                raise ValueError("invalid param got when using {} to regxp ".format(v))
+        
+        return conn_args
+
     def get_connection(self):
         return self.connector
 
@@ -39,12 +86,10 @@ class SimpleConnector(SqlGenerator):
             conn = self.get_connection()
             return conn.cursor(DictCursor) if cursor_type == 'dict' else conn.cursor()
         except Exception as e:
-            self.logger.error(str(e))
             self.logger.warning("ping and reconnect ...")
             conn.ping(reconnect=True)
             return conn.cursor(DictCursor) if cursor_type == 'dict' else conn.cursor()
 
-
     def execute_sql(self, sql, param=None, cursor_type=None):
         """
         # cursor_type为游标类型（默认返回值为元祖类型），可选字典游标，将返回数据的字典形式
@@ -120,21 +165,17 @@ class SimpleConnector(SqlGenerator):
         sql, param = self.generate_select_sql(*args, **kwargs)
         return self.execute_sql(sql, param)
 
-    def insert_into(self, table, data: dict or list, columns_order=None, ignroe=False, on_duplicate_key_update: str = None,return_id=False):
+    def insert_into(self, table, data: dict or list, columns_order=None, ignroe=False, on_duplicate_key_update: str = None, return_id=False):
         """
         此方法将返回 插入后的 id
         """
-        sql, param = self.generate_insert_sql(table,data,columns_order,ignroe,on_duplicate_key_update)
+        sql, param = self.generate_insert_sql(table, data, columns_order, ignroe, on_duplicate_key_update)
         return self.execute_with_return_id(sql, param) if return_id else self.execute_sql(sql, param)[1]
 
-    def replace(self, *args, **kwargs):
+    def replace_into(self, *args, **kwargs):
         sql, param = self.generate_replace_into_sql(*args, **kwargs)
         return self.execute_sql(sql, param)[1]
 
-    def replace_into(self,*args, **kwargs):
-        sql, param = self.generate_replace_into_sql(*args, **kwargs)
-        return  self.execute_sql(sql, param)[1]
-
     def update_by_primary(self, *args, **kwargs):
         sql, param = self.generate_update_sql_by_primary(*args, **kwargs)
         return self.execute_sql(sql, param)[1]
@@ -155,6 +196,13 @@ class SimpleConnector(SqlGenerator):
         return self.execute_sql(sql, param)[1]
 
 
+class MysqlConnector(MysqlSqler, SimpleConnector):
+
+    def merge_into(self, *args, **kwargs):
+        sql, param = self.generate_merge_sql(*args, **kwargs)
+        return self.execute_sql(sql, param)[1]
+
+
 class SimplePoolConnector(SimpleConnector):
 
     def __init__(self, *args, **kwargs):
@@ -164,20 +212,47 @@ class SimplePoolConnector(SimpleConnector):
         return self.connection_pool.connection()
 
 
-class PoolConnector(SimplePoolConnector):
+class MysqlPool(SimplePoolConnector, MysqlConnector):
 
     def __init__(self, host, user, password, database, port=3306, charset='utf8', re_create=True, **kwargs):
         """
-        re_create=True 则会
+        # todo re_create=True 则会
+        mincached 最小的连接数（实例化时最开始就实例化这个数的连接）
+
+        连接断开时，连接池能感受到
         """
-        self.host = host = host
-        self.port = port
-        self.user = user
-        self.database = database
-        self.password = password
-        connargs = {"host": self.host, "user": self.user, "password": self.password, "database": self.database, 'port': self.port, "charset": self.charset,
+        self._host = host = host
+        self._port = port
+        self._user = user
+        self._database = database
+        self._password = password
+        self._charset = charset
+        connargs = {"host": self._host, "user": self._user, "password": self._password, "database": self._database, 'port': self._port, "charset": self._charset,
                     "creator": pymysql, "mincached": 3, "maxcached": 8, "maxshared": 5, "maxconnections": 10, "blocking": True, "maxusage": 0}
 
         connargs.update(kwargs)
 
         self.connection_pool = PooledDB(**connargs)
+
+
+class PostgrePool(SimplePoolConnector):
+
+    def __init__(self, host, user, password, database, port=5432, charset='utf8', re_create=True, **kwargs):
+        """
+        # todo re_create=True 则会
+        mincached 最小的连接数（实例化时最开始就实例化这个数的连接）
+
+        连接断开时，连接池能感受到
+        """
+        self._host = host
+        self._port = port
+        self._user = user
+        self._database = database
+        self._password = password
+        self._charset = charset
+        connargs = {"host": self._host, "user": self._user, "password": self._password, "database": self._database, 'port': self._port, "charset": self._charset,
+                    "creator": pg2, "mincached": 3, "maxcached": 8, "maxshared": 5, "maxconnections": 10, "blocking": True, "maxusage": 0}
+
+        connargs.update(kwargs)
+
+        self.connection_pool = PooledDB(**connargs)
